# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from Image import *
from Tools import *

class CD:
	"chain drawing basics stuff"
	def __init__(self):
		self.startCD = 0
		self.clenCD = 0
		self.strandCD = ''
		self.numCD = 0

def drawCD(r,CD,x,flip,verbose):
	#determine the chain segment values

	#parameters for the chain segment
	#N - number of residues in the chain to be displayed  
	#W - width between the centers of transmembrane segments
	#R - symbol radius 
	#Htot - height or distance between the two membrane lines
	#Hw - height of the working display area
	#B - buffer width of symbol
	#L - number of layers of display 
	#S - number of symbols per layer
	#sl -  size of layer is 3.5

	L = 6
	sl = 3.5
	R = 5.0
	W = 16.0*R
	B = 1.0
	Htot = (7.0*B +6.0*sl)*R
	#Hw = 16.0*R
	ybase = 0.0

	#set up the display area for the chain (number of symbols in the layer
	#first the connection points

	#then the variable layers
	N = CD.clenCD
	if N < 6:
		msg = "membrane spanning region is too small"
		printVerbose(verbose,msg)
	elif N > 42:
		msg = "membrane spanning region is too big"
		printVerbose(verbose,msg)
	else:
		#determine the number of symbols per layer
		S = N/L
		#print "there are %d symbols per layer" % S

		#determine the number of extra symbols
		O = N%L
		#print "there are %d extra symbols in this chain" % O

	#work though the layers filling in data		
  	i = 0
	k = 0
	l5 = 1
	while i <= 5:
		if flip == 1:
			factor = -1
			y = ybase - ((B + .5*sl) + (B + sl)*(5-i))*R
		else:
			factor = 1
			y = ybase - ((B + .5*sl) + (B + sl)*i)*R

		layers = layer_data(O,i,S,R,y,factor)
		num = layers[0]
		incy = layers[1]
		ystart = layers[2]

		xstart = x - (num -1)*R
		j = 1
		while j <= num:
			xd = xstart + (j - 1)*2*R
                       	l1 = CD.startCD + k + (j -1)
                       	l2 = CD.strandCD[k+(j-1):k+j]
			if num == 4 and j == 3:
				ystart = ystart -incy*factor
			if num == 6 and j == 4:
				ystart = ystart - incy*factor
                       	l4 = ystart -(j -1) *incy*factor
			image_data2(r,l1,l2,xd,l4)
			j = j + 1

		k = k + num
		i = i + 1				
	return 

def drawCDpart(r,CD,x,flip,verbose):
	#determine the partial segment values

	#parameters for the partial segment
	#N - number of residues in the chain to be displayed  
	#W - width between the centers of transmembrane segments
	#R - symbol radius 
	#Htot - height or distance between the two membrane lines
	#Hw - height of the working display area
	#B - buffer width of symbol
	#L - number of layers of display 
	#S - number of symbols per layer
	#sl -  size of layer is 3.5

	l5 = 1
	L = 5
	sl = 3.5
	R = 5.0
	W = 16.0*R
	B = 1.0
	Htot = (7.0*B +6.0*sl)*R
	#Hw = 16.0*R
	ybase = 0.0

	#set up the display area for the chain (number of symbols in the layer
	#first the connection points

	#then the variable layers
	N = CD.clenCD
	if N < 6:
		msg = "partial membrane region is too small"
		printVerbose(verbose,msg)
	elif N > 79:
		msg = "partial membrane region is too big"
		printVerbose(verbose,msg)
	elif N > 5 and N < 28:
		if flip == 1:
			y = ybase - ((B+sl)*6.0- 1)*R
			fact = -1
		else:
			y = ybase - (B + 1)*R
			fact = 1

		if N <= 9:
			div = W/N
			#print "w is %d and div is %d" % (W,div)
			j = 1
			k = 0
			out = div - R
			while j <= N:
				l1 = CD.startCD +k
				l2 = CD.strandCD[k:k+1]
				if N == 9:
					l3 = x + j*2.0*R -2.0*R
				else:
					l3 = x + out + (j-1)*div
				l4 = y 
				image_data2(r,l1,l2,l3,l4)
				j = j + 1
				k = k + 1
				
		elif N > 9 and N < 28: 
			#determine the length of possible sides
			side = (N - 7)/2
			odd = (N-7)%2

			#if odd increase side by 1 and reduce cross section by 1
			if odd > 0:
				side = side + 1

			#draw first side
 			j = 0
			k = 0
			while j < side:
				ystart = y -2.0*j*R*fact	
				image_data(r,k,CD,x,ystart)
				j = j + 1
				k = k + 1

			#draw the cross section
			#print "odd is %d" % odd
			if odd > 0:
				num = 6
				off = .286*R
				#print "offset is %d" % off
			else:
				num = 7
				off = 0.0

			j = 1
			xstart = x
			while j <= num:
                        	x = xstart + j*(2.0*R + off)
				image_data(r,k,CD,x,ystart)
                        	j = j + 1
                       		k = k + 1 

			#draw the other side
			x = x + 2.0*R + off
 			j = 1
			while j <= side:
				y = ystart +2.0*(j-1)*R*fact			
				image_data(r,k,CD,x,y)
				j = j + 1
				k = k + 1
		else:
			#determine if even or odd
			even = N/2
			odd = N%2

	else:
		#determine if even or odd
		even = N/2
		odd = N%2
		if odd == 0:
			bottom = 8
			offs = .5*R
		else:
			bottom = 9
			offs = 0.0

		N2 = (N - bottom)/2
		#determine the number of symbols per layer
		S = N2/L
		#print "there are %d symbols per layer" % S

		#determine the number of extra symbols
		O = N2%L
		#print "there are %d extra symbols in this chain" % O

		#work though the layers filling in data		
		i = 0
		k = 0
		l5 = 1
		while i <= 4:
			if flip == 1:
				factor = -1
				y = ybase - ((B + .5*sl) + (B + sl)*(5-i))*R
			else:
				factor = 1
				y = ybase - ((B + .5*sl) + (B + sl)*i)*R

			layers = layer_data(O,i,S,R,y,factor)
			num = layers[0]
			incy = layers[1]
			ystart = layers[2]
			
			xstart = x - (num -1)*R
			j = 1
			while j <= num:
				xd = xstart + (j - 1)*2*R
                       		l1 = CD.startCD + k + (j -1)
                       		l2 = CD.strandCD[k+(j-1):k+j]
                       		l3 = xd
				if num == 4 and j == 3:
					ystart = ystart -incy*factor
				if num == 6 and j == 4:
					ystart = ystart - incy*factor
                       		l4 = ystart -(j -1) *incy*factor
				image_data2(r,l1,l2,l3,l4)
				j = j + 1

			k = k + num
			i = i + 1				
	
		#draw the crossing region
		j = 1
		while j <= bottom:
			xd = x + (j-1)*2*R + offs
			if flip == 1:
				y = ybase - (B + .5*sl)*R
			else:
				y = ybase - ((B + .5*sl) + (B + sl)*5)*R
			image_data(r,k,CD,xd,y)
			k = k + 1
			j = j + 1

		#draw the return to the original side of the membrane
		x = x + W
		i = 0
		l5 = 1

		while i <= 4:
			if flip == 1:
				factor = 1
				y = ybase - ((B + .5*sl) + (B + sl)*(i+1))*R
			else:
				factor = -1
				y = ybase - ((B + .5*sl) + (B + sl)*(5-(i+1)))*R

	 		layers = layer_data(O,i,S,R,y,factor)
			num = layers[0]
			incy = layers[1]
			ystart = layers[2]

			xstart = x - (num -1)*R
			j = 1
			while j <= num:
				xd = xstart + (j - 1)*2*R
				if num == 4 and j == 3:
					ystart = ystart -incy*factor
				if num == 6 and j == 4:
					ystart = ystart - incy*factor
                       		yd = ystart -(j -1) *incy*factor
				image_data(r,k,CD,xd,yd)
				j = j + 1
				k = k + 1

			i = i + 1				

	return 

